//
//  DevicesTableViewController.swift
//  CalibrationTest
//
//  Created by Rocky Lin on 2/25/19.
//  Copyright © 2019 rlin. All rights reserved.
//

import UIKit
import MetaWear

class DevicesTableViewController: UITableViewController {

    var devices = [MetaWear]()
    var isScanning = false
    weak var delegate: DevicesTableViewControllerDelegate?

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        self.setScanning(true)
    }

    func setScanning(_ on: Bool) {
        if on {
            if !isScanning {
                MetaWearScanner.shared.startScan(allowDuplicates: true, callback: { [weak self] device in
                    DispatchQueue.main.async {
                        guard let self = self, self.isScanning else {
                            return
                        }
                        if !self.devices.contains(device) {
                            self.devices.append(device)
                        }
                        self.tableView.reloadData()
                    }
                })
                isScanning = true
            }
        } else {
            if isScanning {
                isScanning = false
                MetaWearScanner.shared.stopScan()
                devices = [MetaWear]()
                tableView.reloadData()
            }
        }
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return devices.count
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)

        setScanning(false)
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        return tableView.dequeueReusableCell(withIdentifier: "Device", for: indexPath)
    }

    override func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        let device = devices[indexPath.row]
        cell.textLabel?.text = device.name
    }

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        delegate?.didSelectDevice(devices[indexPath.row])
        self.dismiss(animated: true, completion: nil)
    }
}

protocol DevicesTableViewControllerDelegate: AnyObject {

    func didSelectDevice(_ device: MetaWear)
}
